<?php
/**
 * Plugin Name: RichDuplicator
 * Description: Allows easy duplication of posts and pages, including all metadata and custom fields.
 * Version: 1.0
 * Author: Felix Krusch
 * Text Domain: rich-duplicator
 * Domain Path: /languages
 */

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}

class RichDuplicator {

    public function __construct() {
        add_action('plugins_loaded', array($this, 'load_textdomain'));
        add_filter('post_row_actions', array($this, 'add_duplicate_link'), 10, 2);
        add_filter('page_row_actions', array($this, 'add_duplicate_link'), 10, 2);
        add_action('admin_action_duplicate_post_as_draft', array($this, 'duplicate_post_as_draft'));
    }

    public function load_textdomain() {
        load_plugin_textdomain('rich-duplicator', false, dirname(plugin_basename(__FILE__)) . '/languages/');
    }

    public function add_duplicate_link($actions, $post) {
        $actions['duplicate'] = '<a href="' . wp_nonce_url('admin.php?action=duplicate_post_as_draft&post=' . $post->ID, 'duplicate_post_' . $post->ID) . '" title="' . esc_attr(__('Duplicate this item', 'rich-duplicator')) . '">' . __('Duplicate', 'rich-duplicator') . '</a>';
        return $actions;
    }

    public function duplicate_post_as_draft() {
        global $wpdb;
        if (! (isset($_GET['action']) && $_GET['action'] == 'duplicate_post_as_draft'))
            return;

        if (!wp_verify_nonce($_GET['_wpnonce'], 'duplicate_post_' . $_GET['post']))
            wp_die(__('Security check failed', 'rich-duplicator'));

        if (!current_user_can('edit_posts'))
            wp_die(__('You do not have sufficient permissions to access this page.', 'rich-duplicator'));

        $post_id = (isset($_GET['post']) ? absint($_GET['post']) : 0);
        $post = get_post($post_id);
        if (isset($post) && $post != null) {
            $args = array(
                'comment_status' => $post->comment_status,
                'ping_status'    => $post->ping_status,
                'post_author'    => $post->post_author,
                'post_content'   => $post->post_content,
                'post_excerpt'   => $post->post_excerpt,
                'post_name'      => $post->post_name . "-copy",
                'post_parent'    => $post->post_parent,
                'post_password'  => $post->post_password,
                'post_status'    => 'draft',
                'post_title'     => $post->post_title . ' Copy',
                'post_type'      => $post->post_type,
                'to_ping'        => $post->to_ping,
                'menu_order'     => $post->menu_order
            );

            $new_post_id = wp_insert_post($args);

            $post_meta_infos = $wpdb->get_results("SELECT meta_key, meta_value FROM $wpdb->postmeta WHERE post_id=$post_id");
            if (count($post_meta_infos) != 0) {
                foreach ($post_meta_infos as $meta_info) {
                    $meta_key = $meta_info->meta_key;
                    if ($meta_key == '_wp_old_slug') continue; // Skip copying of old slugs
                    
                    $meta_value = $meta_info->meta_value;
                    if (is_serialized($meta_value)) { 
                        $meta_value = maybe_unserialize($meta_value);
                    }
                    update_post_meta($new_post_id, $meta_key, $meta_value);
                }
            }

            wp_redirect(admin_url('post.php?action=edit&post=' . $new_post_id));
            exit;
        } else {
            wp_die(__('Post creation failed, could not find original post:', 'rich-duplicator') . ' ' . $post_id);
        }
    }
}

new RichDuplicator();
